//-------------------------------------------------------------------
// PROGRAM: randbern.hpp
//
// PURPOSE: Generates bernoulli random numbers
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula derived from Law & Keltons ( see reference ).
      1. Generate U( 0, 1 )
      2. If U <= probability, return X = 1. Otherwise return X = 0;
         [ Law & Kelton, 1991 ]

   Discrete probability distributions are appropriate models of random
phenomena only if the random variables are measured by counting.

Parameters: P ~ probability of success where outcome x=0 indicates
            failure or outcome x=1 indicates success.

Range:      [ 0 or 1 ]

Mean:       probability

Variance:   probability * ( 1 - probability )

Type:       Probability is the probability of success during a particular
            sequence of trials.

   "Bernoulli trials are chance experiments in which the outcome of 
each trial is expressed as either a success or failure. The P[ success ] = p,
where p is a constant for a particular sequence of trials." [ Bulgren, 1982 ]

References: Discrete System Simulation by William G. Bulgren 1982, pg. 175.

            Simulation Modeling & Analysis by Law and Kelton, 1991, 
            pg., 496-497.
---------------------------------------------------------------------
*/

#if !defined _RANDBERN_HPP
#define      _RANDBERN_HPP 1

//------------------------------ Includes -------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


class RandBernoulli : public Rand
   {

   private:

      double probability;

   public:

      //-- constructors --//
      RandBernoulli( double _probability, char *_name = "-" );
      RandBernoulli( double _probability, UINT _stream, long _seed, 
            char *_name = "-" );

      //-- destructor --//
      virtual ~RandBernoulli( void ) { }

      //-- returns a bernoulli random variable --//
      virtual double RandValue( void );

   };


#endif
