//-------------------------------------------------------------------
// PROGRAM: randbeta.hpp
//
// PURPOSE: Generates beta random numbers
//
// NOTE: A beta( 1, 1 ) is a U( 0, 1 ). The beta is bound between 
//       0 and 1 unless translated to a different range.
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   Generate Y1 ~ gamma( shape1, 1 ) and Y2 ~ gamma( shape2, 1 ) independently.
   Return X = Y1 / ( Y1 + Y2 )


Parameters: both shape parameters are real and > 0.

Range:      [ lowerBound, upperBound ]

Mean:       "of the Beta( 0, 1 ) variate" is: shape1 / ( shape1 + shape2 )

Var:        "of the Beta( 0, 1 ) variate" is: ( shape1 * shape2 ) /
            ( ( shape1 + shape2 )^2 * ( shape1 + shape2 + 1 ) )


   Used as a rough model in the absence of data. The beta is able to
take on a wide variety of shapes. Although the range of the beta is
between ( 0, 1 ), it can be scaled to any upperBound - lowerBound 
range by using the simple formula X = lowerBound + ( upperBound -
lowerBound ) * X', where X' is the beta( 0, 1 ) variate.

References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 338-339, 479-484, and 492-493.

            Discrete System Simulation by William G. Bulgren, 1982,
            pg., 170-171.
---------------------------------------------------------------------
*/

#if !defined _RANDBETA_HPP
#define      _RANDBETA_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif

//----------------------- Forward references ----------------------//

class RandGamma;


class RandBeta : public Rand
   {

   protected:

      double shape1;
      double shape2;
      double upperBound;
      double lowerBound;

      RandGamma *pRNG1;          // these must be independent!
      RandGamma *pRNG2;

      //-- initialize the random number generators --//
      void Initialize( double _shape1, double _shape2,
            UINT _stream, long _seed );

   public:

      //-- constructors --//
      RandBeta( double _shape1, double _shape2, double _lowerBound,
            double _upperBound, char *_name = "-" );
      RandBeta( double _shape1, double _shape2, double _lowerBound,
            double _upperBound, long _seed, char *_name = "-" );
      RandBeta( double _shape1, double _shape2, double _lowerBound,
            double _upperBound, UINT _stream, long _seed,
            char *_name = "-" );

      //-- destructor --//
      virtual ~RandBeta( void );

      //-- return an beta random number --//
      virtual double RandValue( void );

   };


#endif


