//-------------------------------------------------------------------
// PROGRAM: randbino.hpp
//
// PURPOSE: Generates binomial random variate
//
// NOTE:    RandBinomial( successfullTrials, probability )
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   Since the sum of t IID Bernoulli random variables has the 
   Binomial distribution, a simple convolution algorithm produces
   the desired distribution.

   Generate Y1, Y2, ..., Yt as IID Bernoulli( prob ) random variates.
   Then return X = Y1 + Y2 + ... + Yt. [ Law and Kelton, 1991 ]
   The random variable X equals the number of successes in t trials.

   Other methods are discussed in Law and Kelton which deal with 
   the problem of summing a large number of Bernoulli variates. Also,
   one could consider the normal approximation if the number of 
   Bernoulli trials to sum is large. One caveat to the normal
   approximation is the possibility of negative values. For a complete
   discussion see [ Bulgren, 1982 ].
   
Parameters: successfullTrials( t ) is a positive integer, and probability
            ( p ) is 0 < p < 1. 

Range:      { 0, 1, ..., t }

Mean:       t*p or successfulTrials * probability

Var:        t*p( 1 - p )

   The binomial distribution can be used in many situations but one
example to consider is if we let binomial variable X with parameters
Y = number of Yes's, Z = number of No's, and p probability of a yes,
then N = Y + Z where N becomes the random binomial variate.
   "Number of successes in t independent Bernoulli trials with prob-
ability p of success on each trial; number of "defective" items in a
batch of size t; number of items in a batch ( e.g., a group of 
people ) of random size; number of items demanded from an inventory."
[ Law & Kelton, 1991 ]


References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 345-346 and 502.

            Discrete System Simulation by William G. Bulgren, 1982,
            pgs., 176-177.
---------------------------------------------------------------------
*/

#if !defined _RANDBINO_HPP
#define      _RANDBINO_HPP 1

//------------------------------ Includes -------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif

//-------------------------- Forward reference --------------------//

class RandBernoulli;



class RandBinomial : public Rand
   {

   private:

      double probability;              // probability of a success
      ULONG  successfullTrials;

      RandBernoulli *pRNG;             // since we sum Bernoulli's

   public:

      //-- constructors --//
      RandBinomial( ULONG successfullTrials, double _probability,
            char *_name = "-" );

      RandBinomial( ULONG successfullTrials, double _probability,
            UINT _stream, long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~RandBinomial( void );

      //-- returns a binomial random variable --//
      virtual double RandValue( void );

   };


#endif

