//-------------------------------------------------------------------
// PROGRAM: randduni.hpp
//
// PURPOSE: Generates discrete uniform ( 0, 1 ) random numbers
//          or discrete uniform ( lowerBound, upperBound ) random 
//          numbers where lowerBound < upperBound
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   Generate U ~ U( 0, 1 )
   Return X = lowerBound + <[ ( upperBound - lowerBound + 1 ) * U ]>
   where we take the largest integer between the <[]>.

Parameters: lowerBound and upperBound are integers.

Range:      { i, i + 1, ...., j }

Mean:       ( lowerBound + upperBound ) / 2

Var:        ( ( upperBound - lowerBound + 1 )^2 - 1 ) / 12

   "Random occurrences with several possible outcomes, each of which
is equally likely; used as a "first" model for a quantity that is
varying among the integers i through j but about which little else is
known." [ Law & Kelton, 1991 ]


References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 344-345 and 497.
---------------------------------------------------------------------
*/

#if !defined _RANDDUNI_HPP
#define      _RANDDUNI_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


class RandDiscreteUniform : public Rand
   {

   private:

      //-- the discrete bounds for the uniform variate --//
      long lowerBound;
      long upperBound;

      long largestInt;           // algorithm coefficient

   public:

      //-- constructors --//
      RandDiscreteUniform( char *_name = "-" );
      RandDiscreteUniform( UINT _stream, long _seed, char *_name = "-" );
      RandDiscreteUniform( long _lowerBound, long _upperBound, long _seed, 
            char *_name = "-" );
      RandDiscreteUniform( long _lowerBound, long _upperBound, \
            UINT _stream, long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~RandDiscreteUniform( void ) { }

      //-- return a discrete uniform number --//
      virtual double RandValue( void );
      
   };

#endif

