//-------------------------------------------------------------------
// PROGRAM: randerla.hpp
//
// PURPOSE: Generates erlang distributed random numbers
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula derived from Law & Keltons ( see reference ), incorrect
   formula. "If X is an m-Erlang random variable with mean Beta, we can
   write X = Y1 + Y2 + ... + Ym, where Yi's are IID exponential random 
   variables, each with mean ( Beta / m )." [ Law and Kelton, 1991 ]

   It should be noted that the m-Erlang is a special case of the gamma 
distribution ( with shape parameter alpha equal to the integer m ).

Parameters: If X1, X2, ..., Xm are independent exponential samples, then
            the sum of these m samples has an Erlang-m distribution. The
            mean Beta of the exponential distribution and the number of 
            exponential samples m are parameters of the distribution.

Range:      [ 0, +infinity ]

Mean:       m * Beta

Variance:   m * Beta^2

Type:       Beta is positive real and m is a positive integer.

   "Erlang distribution is used in situations in which an activity occurs
in phases and each phase has an exponential distribution. For large m the
Erlang approaches the normal distribution. The Erlang distribution is often
used to represent the time required to complete a task. The Erlang
distribution is a special case of the gamma distribution in which the shape
parameter, Alpha, is an integer". [ Pegden et al, 1990 ]

References: Simulation Modeling & Analysis by Averill M. Law and W. David
            Kelton, 1991, pg., 486.

            Introduction to Simulation using SIMAN by C. Dennis Pegden, 
            Robert E. Shannon, and Randall P. Sadowski, 1990, pg., 562.
---------------------------------------------------------------------
*/

#if !defined _RANDERLA_HPP
#define      _RANDERLA_HPP 1

//---------------------------- Includes ---------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


class RandErlang : public Rand
   {

   private:

      UINT shape;             // number of samples
      double scale;           // exponential mean

      double a;               // algorithm coefficients, See Law & Kelton

   public:

      //-- constructors --//
      RandErlang( UINT _shape, double _scale, char *_name = "-" );
      RandErlang( UINT _shape, double _scale, UINT _stream,
            long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~RandErlang( void ) { }

      //-- returns an erlang random number --//
      virtual double RandValue( void );

   };


#endif


