//-------------------------------------------------------------------
// PROGRAM: randexp.hpp
//
// PURPOSE: Generates exponential random numbers
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Inverse transform method where X = - mean * ( ln( 1 - U( 0,1 ) ) )
   Which is the same as X = -mean * ( ln( U( 0,1 ) ); [ Bulgren 1982 ]
   Beta = 1 / rate term lamda [ Thesen and Travis, 1992 ]

Parameters: mean is the rate parameter and is > 0.

Range:      [ 0, +infinity ]

Mean:       Beta ( mean )

Variance:   Beta ^ 2

   "This distribution is often used to model random arrival and breakdown
processes, but it is generally inappropriate for modeling process delay
times. The exponential distribution is used in situations in which the 
random quantities satisfy the memoryless property." [ Pegden et al, 1990 ]


Reference: Discrete System Simulation by William G. Bulgren 1982, pg. 168.
           
           Simulation for Decision Making by Arne Thesen and 
           Laurel E. Travis, 1992, pg. 24.

           Introduction to Simulation Using SIMAN by C. Dennis Pegden, 
           Robert E. Shannon, and Randall P. Sadowski, 1990, pg. 561.
---------------------------------------------------------------------
*/

#if !defined _RANDEXP_HPP
#define      _RANDEXP_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


class RandExponential : public Rand
   {

   private:

      double mean;           // rate parameter

   public:

      //-- constructors --//
      RandExponential( double _mean, char *_name = "-" );
      RandExponential( double _mean, long _seed, char *_name = "-" );
      RandExponential( double _mean, UINT _stream, long _seed, 
            char *_name = "-" ); 

      //-- destructor --//
      virtual ~RandExponential( void ) { }
 
      //-- return an exponential random number --//
      virtual double RandValue( void );

   };


#endif
