//-------------------------------------------------------------------
//  PROGRAM: randgama.cpp
//-------------------------------------------------------------------

#pragma hdrstop

//--------------------------- Includes ----------------------------//

#if !defined ( _RANDGAMA_HPP )
#include "randgama.hpp"
#endif

#if !defined ( _RANDEXP_HPP )
#include "randexp.hpp"
#endif

#if !defined ( _RANDUNIF_HPP )
#include "randunif.hpp"
#endif


extern "C"
{
#include <math.h>
#include <string.h>
#include <stdio.h>
}



//-- constructors --//

RandGamma::RandGamma( double _shape, double _scale, char *_name )
   : Rand( 1, _name ), 
     shape( fabs( _shape ) ),
     scale( fabs( _scale ) ),
     a( 0 ),
     b( 0 ),
     q( 0 ),
     phi( 0 ),
     d( 0 ),

     pRNG_Exp( NULL ),
     pRNG_One( NULL ),
     pRNG_Two( NULL )
   {
   SetDistName( "Gamma" );

   //-- set the coefficients --//
   char temp[ 40 ];
   sprintf( temp, "%g", shape );
   SetParameterA_Name( temp );
   sprintf( temp, "%g", scale );
   SetParameterB_Name( temp );

   Initialize( 1, 0 );     // default stream, default seed
   }

RandGamma::RandGamma( double _shape, double _scale, long _seed, char *_name )
   : Rand( 1, _seed, _name ),
     shape( fabs( _shape ) ),
     scale( fabs( _scale ) ),
     a( 0 ),
     b( 0 ),
     q( 0 ),
     phi( 0 ),
     d( 0 ),

     pRNG_Exp( NULL ),
     pRNG_One( NULL ),
     pRNG_Two( NULL )
   {
   SetDistName( "Gamma" );

   //-- set the coefficients --//
   char temp[ 40 ];
   sprintf( temp, "%g", shape );
   SetParameterA_Name( temp );
   sprintf( temp, "%g", scale );
   SetParameterB_Name( temp );

   Initialize( 1, _seed );    // default stream
   }

RandGamma::RandGamma( double _shape, double _scale, UINT _stream,
   long _seed, char *_name )
   : Rand( _stream, _seed, _name ),
     shape( fabs( _shape ) ),
     scale( fabs( _scale ) ),
     a( 0 ),
     b( 0 ),
     q( 0 ),
     phi( 0 ),
     d( 0 ),

     pRNG_Exp( NULL ),
     pRNG_One( NULL ),
     pRNG_Two( NULL )
   {
   SetDistName( "Gamma" );

   //-- set the coefficients --//
   char temp[ 40 ];
   sprintf( temp, "%g", shape );
   SetParameterA_Name( temp );
   sprintf( temp, "%g", scale );
   SetParameterB_Name( temp );

   Initialize( _stream, _seed );
   }


RandGamma::~RandGamma( void )
   {
   if ( pRNG_Exp )
      delete pRNG_Exp;

   if ( pRNG_One )
      delete pRNG_One;

   if ( pRNG_Two )
      delete pRNG_Two;
   }


void RandGamma::Initialize( UINT _stream, long _seed )
   {
   if ( shape < 1 )
      InitializeLessThanOne( _stream, _seed );

   else if ( shape == 1 )
      InitializeEqualOne( _stream, _seed );

   else
      InitializeGreaterThanOne( _stream, _seed );

   return;
   }


//-- return an gamma random variable --//
double RandGamma::RandValue( void )
   {
   AddOneObservation();

   //-- use the appropriate method based on initialization --//
   double gammaOne = (this->*pRandFn)();  // returns gamma( shape, 1 )

   return ( scale * gammaOne );
   }


void RandGamma::InitializeGreaterThanOne( UINT _stream, long _seed )
   {
   //-- shape( shape ) is greater than one --//

   a = 1 / sqrt( 2 * shape - 1 );
   b = shape - log( 4 );
   q = shape + 1 / a;
   phi = 4.5;
   d = 1 + log( phi );

   //-- set the function pointer to the correct method --//
   pRandFn = (RANDPROC) &RandGamma::GreaterThanOne;

   //-- new the Uniform generators --//
   pRNG_One = new RandUniform( _stream, _seed );
   pRNG_Two = new RandUniform( _stream + 1, _seed );

   return;
   }

double RandGamma::GreaterThanOne( void )
   {
   //-- shape( shape ) is greater than one --//

   while( 1 )
      {
      double U_One = pRNG_One->RandValue();
      double U_Two = pRNG_Two->RandValue();

      double V = a * log( U_One / ( 1 - U_One ) );
      double Y = shape * exp( V );
      double Z = U_One * U_One * U_Two;
      double W = b + q * V - Y;

      double temp1 = W + d - phi * Z;
      double temp2 = log( Z );

      if ( temp1 >= 0 || W >= temp2 )
         return Y;

      } // end of WHILE

   }

void RandGamma::InitializeLessThanOne( UINT _stream, long _seed )
   {
   //-- shape( alpha ) is < 1 --//

   b = ( exp( 1 ) + shape ) / exp( 1 );

   //-- set the function pointer to the correct method --//
   pRandFn = (RANDPROC) &RandGamma::LessThanOne;

   //-- new the Uniform generators --//
   pRNG_One = new RandUniform( _stream, _seed );
   pRNG_Two = new RandUniform( _stream + 1, _seed );

   return;
   }

double RandGamma::LessThanOne( void )
   {
   //-- shape( alpha ) is less than one --//

   while( 1 )
      {
      double U_One = pRNG_One->RandValue();
      double P = b * U_One;

      if ( P <= 1 )
         {
         double Y = pow( P, ( 1/shape ) );
         double U_Two = pRNG_Two->RandValue();

         if ( U_Two <= exp( -Y ) )
            return Y;
         }

      else
         {
         double Y = - log( ( b - P ) / shape );
         double U_Two = pRNG_Two->RandValue();

         if ( U_Two <= pow( Y, ( shape - 1 ) ) )
            return Y;
         }

      } // end of WHILE

   }


void RandGamma::InitializeEqualOne( UINT _stream, long _seed )
   {
   //-- set the function pointer to the correct method --//
   pRandFn = (RANDPROC) &RandGamma::EqualOne;

   //-- new the Exponential generator --//
   pRNG_Exp = new RandExponential( 1, _stream, _seed );

   return;
   }

double RandGamma::EqualOne( void )
   {
   //-- shape( alpha ) is == one --//
   //   same as exponential        //

   return ( pRNG_Exp->RandValue() );
   }

