//-------------------------------------------------------------------
// PROGRAM: randgama.hpp
//
// PURPOSE: Generates gamma random numbers
//
// NOTE: If shape( alpha ) = 1 use exponential, if shape is positive
//       integer use Erlang. To generate Chi-square with "v" degrees
//       of freedom, use shape = ( v/2 ) and scale( beta ) = 2.
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   There are three cases to consider and all depend on shape ( alpha ).
   case 1: shape < 1    Aherns & Dieter ( 1974 )
   case 2: shape = 1    Exponential with mean scale ( beta )
   case 3: shape > 1    Cheng ( 1977 )


Parameters: both shape parameters are real and > 0.

Range:      [ 0, +infinity ]

Mean:       shape * scale

Var:        shape * scale * scale


   "Time to complete some task, e.g., customer service or machine repair"
   [ Law & Kelton, 1991 ]


References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 331-333, 487-490, and 518.

            Discrete System Simulation by William G. Bulgren, 1982,
            pg., 169-170.
---------------------------------------------------------------------
*/

#if !defined _RANDGAMA_HPP
#define      _RANDGAMA_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif

#include <math.h>


//----------------------- Forward references ----------------------//

class RandGamma;
class RandExponential;
class RandUniform;


//---------------------------- Typedefs ---------------------------//

//-- gamma random value generation method --//
//   this points to one of three possible functions
typedef double (RandGamma::*RANDPROC)( void );


class RandGamma : public Rand
   {

   private:

      double shape;           // alpha parameter
      double scale;           // beta  parameter

      //-- Cheng (1977) for alpha > 1 --//
      double a;               // a = 1 / sqrt( 2 * alpha - 1 )
      double b;               // b = alpha - ln 4
      double q;               // q = alpha + 1 / a
      double phi;             // phi = 4.5
      double d;               // d = 1 + ln( phi )

      //-- Ahrens and Dieter (1974) is acceptance - rejection technique --//
                              // b = ( e + alpha ) / e
      
      //-- initialize the gamma generator --//
      void Initialize( UINT _stream, long _seed );
      void InitializeGreaterThanOne( UINT _stream, long _seed );
      void InitializeLessThanOne( UINT _stream, long _seed );
      void InitializeEqualOne( UINT _stream, long _seed );

      //-- get a random gamma value based on alpha --//
      double GreaterThanOne( void );
      double LessThanOne( void );
      double EqualOne( void );

      //-- pointer to the random generator method --//
      RANDPROC pRandFn;


   protected:

      RandExponential *pRNG_Exp;    // for alpha == 1

      //-- these will use different streams --//
      RandUniform     *pRNG_One;    // otherwise
      RandUniform     *pRNG_Two;    // otherwise

   public:

      //-- constructors --//
      RandGamma( double _shape, double _scale, char *_name = "-" );
      RandGamma( double _shape, double _scale, long _seed, char *_name = "-" );
      RandGamma( double _shape, double _scale, UINT _stream, long _seed, 
            char *_name = "-" ); 

      //-- destructor --//
      virtual ~RandGamma( void );

      //-- return an gamma random number --//
      virtual double RandValue( void );

   };


#endif

