 //-------------------------------------------------------------------
// PROGRAM: randnbin.hpp
//
// PURPOSE: Generates negative binomial random numbers
//
// NOTE:    Number of failures before the sth success in a sequence of
//          independent bernoulli trials with probability p of success 
//          on each trial.
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula derived from Law & Kelton, see reference.
   Generate Y1, Y2, ..., Ys as IID geometric( prob ) variates.
   Return X = Y1 + Y2 + ... + Ys.

   For large sequences of numbers to sum, one should consider the
   consider the formula from Fishman which utilizes the relationship
   between the negative binomial and the Gamma & Poisson distributions.
   [ Fishman, 1973 ]

Parameters: numSuccesses is a positive integer, probability is between
            ( 0, 1 ).

Range:      { 0, 1, ... }

Mean:       numSuccesses( 1 - probability ) / probability

Var:        numSuccesses( 1 - probability ) / ( probability * probability )


   "Number of failures before the sth success in a sequence of independent
Bernoulli trials with probability p of success on each trial; number of
good items inspected before encountering the sth defective item; number
of items in a batch of random size; number of items demanded from an 
inventory." [ Law & Kelton, 1991 ]
   

References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 348-349 and 502-503.

            Discrete System Simulation by William G. Bulgren, 1982,
            pg., 176.

            Concepts and Methods in Discrete Event Digital Simulation
            by George S. FishMan, 1973, pgs., 226-227.
---------------------------------------------------------------------
*/

#if !defined _RANDNBIN_HPP
#define      _RANDNBIN_HPP 1

//------------------------------ Includes -------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif

//-------------------------- Forward reference --------------------//

class RandGeometric;



class RandNegBinomial : public Rand
   {

   private:

      double probability;
      ULONG  numSuccesses;

      RandGeometric *pRNG;

   public:

      //-- constructors --//
      RandNegBinomial( ULONG numSuccesses, double _probability,
            char *_name = "-" );
      RandNegBinomial( ULONG numSuccesses, double _probability,
            UINT _stream, long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~RandNegBinomial( void );

      //-- returns a bernoulli random variable --//
      virtual double RandValue( void );

   };


#endif

