//-------------------------------------------------------------------
// PROGRAM: randtri.hpp
//
// PURPOSE: Generates triangular distributed random numbers
//
// NOTE:    Composition method
//          tMin <= tMode < tMax     or     tMin < tMode <= tMax
//          The constructor cannot check that this is valid !!
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Inverse transform method: 
   If X ~ triangular( 0, 1, ( max - min ) / ( mode - min ) )
   then X' = min + ( mode - min ) * X   is equivalent to
   triang( min, mode, max ) [ Law & Kelton, 1982 ]

   Formula obtained from Hoover & Perry, see reference. Composition
   method.

   Note: There may be a problem with independence in this approach!
         ie., I'm not sure if my samples are independent.


Parameters: The min, mode, and max values for the distribution
            specified as real numbers with a min < mode < max

Range:      [ min, max ]

Mean: ( min + mode + max ) / 3

Var: ( min^2 + mode^2 + max^2 - mode * min - min * max - mode * max ) / 18

   "The triangular distribution is commonly used in situations in which
the exact form of the distribution is not known, but estimates for the
minimum, maximum, and most likely values are available. The triangular
distribution is easier to use and explain than other distributions that
may be used in this situation ( e.g., the beta distribution )". 
[ Pegden et al, 1990 ]

Reference: Simulation Modeling and Analysis by Averill M. Law and W. David 
           Kelton, 1982, pg., 261

           Introduction to Simulation Using SIMAN by C. Dennis Pegden, 
           Robert E. Shannon, and Randall P. Sadowski, 1990, pg., 566.

           Simulation by Stewart V. Hoover and Ronald F. Perry, 1989,
           pg., 271-272.
---------------------------------------------------------------------
*/

#if !defined _RANDTRI_HPP
#define      _RANDTRI_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


class RandTriangular : public Rand
   {

   private:

      //-- parameters --//
      double tMin;
      double tMode;
      double tMax;

      double S;               // algorithm coefficients; see Simulation by
      double T;               // [ Hoover and Perry, 1989 ]

   public:

      //-- constructors --//
      RandTriangular( double _tMin, double _tMode, double _tMax, 
            char *_name = "-" );
      RandTriangular( double _tMin, double _tMode, double _tMax,
         UINT _stream, long _seed, char *_name = "-" ); 

      //-- destructor --//
      virtual ~RandTriangular( void ) { }
 
      //-- returns a triangular random number --//
      virtual double RandValue( void );

   };

#endif

