//-------------------------------------------------------------------
// PROGRAM: rand_f.hpp
//
// PURPOSE: Generates "F" distributed random numbers
//          with ( k1, k2 ) degrees of freedom
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   If Z1 ~ Chi^2 with k1 df, and Z2 ~ Chi^2 with k2 df, and Z1 and Z2
   are independent, X = ( Z1 / k1 ) / ( Z2 / k2 ) is said to have an
   F distribution with ( k1, k2 ) degrees of freedom. We can write
   X ~ F(k1,k2).


Parameters: k1 and k2 are positive integers that are referred to as the
            degrees of freedom.

Range:      [ 0 , +infinity ]

Mean:       k2 / ( k2 - 2 ),  where k2 > 2

Var:        2 * k2 * k2 * ( k1 + k2 - 2 ) / k1 * ( k2 - 2 )^2 * ( k2 - 4 )


   The F distribution is sometimes referred to as the variance ratio.


References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 485, 682.

            Discrete System Simulation by William G. Bulgren, 1982,
            pg., 173.

            Statisticsl Distributions bye Hastings and Peacock, 1975,
            pgs., 64-67.
---------------------------------------------------------------------
*/

#if !defined _RAND_F_HPP
#define      _RAND_F_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


//------------------------- Forward reference ---------------------//

class RandChiSquare;



class Rand_F : public Rand
   {

   private:

      RandChiSquare  *pRNG1;        // with k1 df
      RandChiSquare  *pRNG2;        // with k2 df

      UINT k1;                      // k1 degrees of freedom
      UINT k2;                      // k2 degrees of freedom

   public:

      //-- constructor --//
      Rand_F( UINT _k1, UINT _k2, UINT _stream, long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~Rand_F( void );

      //-- returns a F distributed random variable --//
      virtual double RandValue( void );

   };

#endif




