//-------------------------------------------------------------------
// PROGRAM: rand_t.hpp
//
// PURPOSE: Generates Students "t" distributed random numbers
//          with k degrees of freedom
// 
// NOTE:    We make use of the gamma to get Chi-squared variables
//-------------------------------------------------------------------

/*
---------------------------------------------------------------------
   Note: Formula obtained from Law & Kelton, see reference.
   If X ~ N( 0, 1 ) and Y has a Chi^2 with k degrees of freedom, then
   X' ~ X / sqrt( Y / k ) has a t distribution with k degrees of 
   freedom.


Parameters: k degreesOfFreedom

Range:      [ -infinity, +infinity ]

Mean:       0

Var:        k / ( k - 2 ),  where k > 2


References: Simulation Modeling & Analysis by Law and Kelton, 1991,
            pgs., 288, 291, 336, 485, 560.

            Discrete System Simulation by William G. Bulgren, 1982,
            pg., 173.

            Statisticsl Distributions bye Hastings and Peacock, 1975,
            pgs., 120-123.
---------------------------------------------------------------------
*/

#if !defined _RAND_T_HPP
#define      _RAND_T_HPP 1

//----------------------------- Includes --------------------------//

#if !defined ( _RAND_HPP )
#include "rand.hpp"
#endif


//------------------------ Forward references ---------------------//

class RandGamma;
class RandNormal;



class Rand_T : public Rand
   {

   private:

      RandGamma      *pRNGGamma;    // with k df
      RandNormal     *pRNGNorm;     // Normal( 0, 1 )

      UINT degreesOfFreedom;        // k df

   public:

      //-- constructor --//
      Rand_T( UINT _degreesOfFreedom, long _seed, char *_name = "-" );

      //-- destructor --//
      virtual ~Rand_T( void );

      //-- returns a log normal random variable --//
      virtual double RandValue( void );

   };

#endif

